@@ -1,17 +1,10 @@
-"""
-    sphinx.builders
-    ~~~~~~~~~~~~~~~
-
-    Builder superclass for all builders.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Builder superclass for all builders."""
 
 import pickle
 import time
 from os import path
-from typing import Any, Dict, Iterable, List, Sequence, Set, Tuple, Union
+from typing import (TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Sequence, Set, Tuple,
+                    Type, Union)
 
 from docutils import nodes
 from docutils.nodes import Node
@@ -40,10 +33,7 @@ try:
 except ImportError:
     multiprocessing = None
 
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
-
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
 
 
@@ -66,18 +56,18 @@ class Builder:
 
     #: default translator class for the builder.  This can be overridden by
     #: :py:meth:`app.set_translator()`.
-    default_translator_class = None  # type: Type[nodes.NodeVisitor]
+    default_translator_class: Type[nodes.NodeVisitor] = None
     # doctree versioning method
     versioning_method = 'none'
     versioning_compare = False
-    # allow parallel write_doc() calls
+    #: allow parallel write_doc() calls
     allow_parallel = False
     # support translation
     use_message_catalog = True
 
     #: The list of MIME types of image formats supported by the builder.
     #: Image files are searched in the order in which they appear here.
-    supported_image_types = []  # type: List[str]
+    supported_image_types: List[str] = []
     #: The builder supports remote images or not.
     supported_remote_images = False
     #: The builder supports data URIs or not.
@@ -90,18 +80,18 @@ class Builder:
         self.doctreedir = app.doctreedir
         ensuredir(self.doctreedir)
 
-        self.app = app              # type: Sphinx
-        self.env = None             # type: BuildEnvironment
-        self.events = app.events    # type: EventManager
-        self.config = app.config    # type: Config
-        self.tags = app.tags        # type: Tags
+        self.app: Sphinx = app
+        self.env: Optional[BuildEnvironment] = None
+        self.events: EventManager = app.events
+        self.config: Config = app.config
+        self.tags: Tags = app.tags
         self.tags.add(self.format)
         self.tags.add(self.name)
         self.tags.add("format_%s" % self.format)
         self.tags.add("builder_%s" % self.name)
 
         # images that need to be copied over (source -> dest)
-        self.images = {}  # type: Dict[str, str]
+        self.images: Dict[str, str] = {}
         # basename of images directory
         self.imagedir = ""
         # relative path to image directory from current docname (used at writing docs)
@@ -109,7 +99,7 @@ class Builder:
 
         # these get set later
         self.parallel_ok = False
-        self.finish_tasks = None  # type: Any
+        self.finish_tasks: Any = None
 
     def set_environment(self, env: BuildEnvironment) -> None:
         """Store BuildEnvironment object."""
@@ -117,7 +107,7 @@ class Builder:
         self.env.set_versioning_method(self.versioning_method,
                                        self.versioning_compare)
 
-    def get_translator_class(self, *args: Any) -> "Type[nodes.NodeVisitor]":
+    def get_translator_class(self, *args: Any) -> Type[nodes.NodeVisitor]:
         """Return a class of translator."""
         return self.app.registry.get_translator_class(self)
 
@@ -178,7 +168,7 @@ class Builder:
     def post_process_images(self, doctree: Node) -> None:
         """Pick the best candidate for all image URIs."""
         images = ImageAdapter(self.env)
-        for node in doctree.traverse(nodes.image):
+        for node in doctree.findall(nodes.image):
             if '?' in node['candidates']:
                 # don't rewrite nonlocal image URIs
                 continue
@@ -219,7 +209,8 @@ class Builder:
         for catalog in status_iterator(catalogs, __('writing output... '), "darkgreen",
                                        len(catalogs), self.app.verbosity,
                                        stringify_func=cat2relpath):
-            catalog.write_mo(self.config.language)
+            catalog.write_mo(self.config.language,
+                             self.config.gettext_allow_fuzzy_translations)
 
     def compile_all_catalogs(self) -> None:
         repo = CatalogRepository(self.srcdir, self.config.locale_dirs,
@@ -228,7 +219,7 @@ class Builder:
         self.compile_catalogs(set(repo.catalogs), message)
 
     def compile_specific_catalogs(self, specified_files: List[str]) -> None:
-        def to_domain(fpath: str) -> str:
+        def to_domain(fpath: str) -> Optional[str]:
             docname = self.env.path2doc(path.abspath(fpath))
             if docname:
                 return docname_to_domain(docname, self.config.gettext_compact)
@@ -264,8 +255,7 @@ class Builder:
         # relative to the source directory and without source_suffix.
         dirlen = len(self.srcdir) + 1
         to_write = []
-        suffixes = None  # type: Tuple[str]
-        suffixes = tuple(self.config.source_suffix)  # type: ignore
+        suffixes: Tuple[str] = tuple(self.config.source_suffix)  # type: ignore
         for filename in filenames:
             filename = path.normpath(path.abspath(filename))
             if not filename.startswith(self.srcdir):
@@ -416,9 +406,9 @@ class Builder:
         else:
             self._read_serial(docnames)
 
-        if self.config.master_doc not in self.env.all_docs:
-            raise SphinxError('master file %s not found' %
-                              self.env.doc2path(self.config.master_doc))
+        if self.config.root_doc not in self.env.all_docs:
+            raise SphinxError('root file %s not found' %
+                              self.env.doc2path(self.config.root_doc))
 
         for retval in self.events.emit('env-updated', self.env):
             if retval is not None:
@@ -520,7 +510,7 @@ class Builder:
             for tocdocname in self.env.files_to_rebuild.get(docname, set()):
                 if tocdocname in self.env.found_docs:
                     docnames.add(tocdocname)
-        docnames.add(self.config.master_doc)
+        docnames.add(self.config.root_doc)
 
         with progress_message(__('preparing documents')):
             self.prepare_writing(docnames)
@@ -564,7 +554,7 @@ class Builder:
         for chunk in status_iterator(chunks, __('writing output... '), "darkgreen",
                                      len(chunks), self.app.verbosity):
             arg = []
-            for i, docname in enumerate(chunk):
+            for docname in chunk:
                 doctree = self.env.get_and_resolve_doctree(docname, self)
                 self.write_doc_serialized(docname, doctree)
                 arg.append((docname, doctree))
