@@ -1,19 +1,12 @@
-"""
-    sphinx.ext.autodoc.importer
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-    Importer utilities for autodoc
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Importer utilities for autodoc"""
 
 import importlib
 import traceback
 import warnings
-from typing import Any, Callable, Dict, List, Mapping, NamedTuple, Optional, Tuple
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
 
-from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
+from sphinx.deprecation import RemovedInSphinx50Warning
+from sphinx.ext.autodoc.mock import ismock, undecorate
 from sphinx.pycode import ModuleAnalyzer, PycodeError
 from sphinx.util import logging
 from sphinx.util.inspect import (getannotations, getmro, getslots, isclass, isenumclass,
@@ -23,11 +16,13 @@ if False:
     # For type annotation
     from typing import Type  # NOQA
 
+    from sphinx.ext.autodoc import ObjectMember
+
 logger = logging.getLogger(__name__)
 
 
 def mangle(subject: Any, name: str) -> str:
-    """mangle the given name."""
+    """Mangle the given name."""
     try:
         if isclass(subject) and name.startswith('__') and not name.endswith('__'):
             return "_%s%s" % (subject.__name__, name)
@@ -38,7 +33,7 @@ def mangle(subject: Any, name: str) -> str:
 
 
 def unmangle(subject: Any, name: str) -> Optional[str]:
-    """unmangle the given name."""
+    """Unmangle the given name."""
     try:
         if isclass(subject) and not name.endswith('__'):
             prefix = "_%s__" % subject.__name__
@@ -105,7 +100,14 @@ def import_object(modname: str, objpath: List[str], objtype: str = '',
             logger.debug('[autodoc] getattr(_, %r)', attrname)
             mangled_name = mangle(obj, attrname)
             obj = attrgetter(obj, mangled_name)
-            logger.debug('[autodoc] => %r', obj)
+
+            try:
+                logger.debug('[autodoc] => %r', obj)
+            except TypeError:
+                # fallback of failure on logging for broken object
+                # refs: https://github.com/sphinx-doc/sphinx/issues/9095
+                logger.debug('[autodoc] => %r', (obj,))
+
             object_name = attrname
         return [module, parent, object_name, obj]
     except (AttributeError, ImportError) as exc:
@@ -141,7 +143,10 @@ def get_module_members(module: Any) -> List[Tuple[str, Any]]:
     """Get members of target module."""
     from sphinx.ext.autodoc import INSTANCEATTR
 
-    members = {}  # type: Dict[str, Tuple[str, Any]]
+    warnings.warn('sphinx.ext.autodoc.importer.get_module_members() is deprecated.',
+                  RemovedInSphinx50Warning)
+
+    members: Dict[str, Tuple[str, Any]] = {}
     for name in dir(module):
         try:
             value = safe_getattr(module, name, None)
@@ -150,31 +155,17 @@ def get_module_members(module: Any) -> List[Tuple[str, Any]]:
             continue
 
     # annotation only member (ex. attr: int)
-    try:
-        for name in getannotations(module):
-            if name not in members:
-                members[name] = (name, INSTANCEATTR)
-    except AttributeError:
-        pass
+    for name in getannotations(module):
+        if name not in members:
+            members[name] = (name, INSTANCEATTR)
 
     return sorted(list(members.values()))
 
 
-Attribute = NamedTuple('Attribute', [('name', str),
-                                     ('directly_defined', bool),
-                                     ('value', Any)])
-
-
-def _getmro(obj: Any) -> Tuple["Type", ...]:
-    warnings.warn('sphinx.ext.autodoc.importer._getmro() is deprecated.',
-                  RemovedInSphinx40Warning)
-    return getmro(obj)
-
-
-def _getannotations(obj: Any) -> Mapping[str, Any]:
-    warnings.warn('sphinx.ext.autodoc.importer._getannotations() is deprecated.',
-                  RemovedInSphinx40Warning)
-    return getannotations(obj)
+class Attribute(NamedTuple):
+    name: str
+    directly_defined: bool
+    value: Any
 
 
 def get_object_members(subject: Any, objpath: List[str], attrgetter: Callable,
@@ -185,7 +176,7 @@ def get_object_members(subject: Any, objpath: List[str], attrgetter: Callable,
     # the members directly defined in the class
     obj_dict = attrgetter(subject, '__dict__', {})
 
-    members = {}  # type: Dict[str, Attribute]
+    members: Dict[str, Attribute] = {}
 
     # enum members
     if isenumclass(subject):
@@ -207,7 +198,7 @@ def get_object_members(subject: Any, objpath: List[str], attrgetter: Callable,
 
             for name in __slots__:
                 members[name] = Attribute(name, True, SLOTSATTR)
-    except (AttributeError, TypeError, ValueError):
+    except (TypeError, ValueError):
         pass
 
     # other members
@@ -223,13 +214,10 @@ def get_object_members(subject: Any, objpath: List[str], attrgetter: Callable,
 
     # annotation only member (ex. attr: int)
     for i, cls in enumerate(getmro(subject)):
-        try:
-            for name in getannotations(cls):
-                name = unmangle(cls, name)
-                if name and name not in members:
-                    members[name] = Attribute(name, i == 0, INSTANCEATTR)
-        except AttributeError:
-            pass
+        for name in getannotations(cls):
+            name = unmangle(cls, name)
+            if name and name not in members:
+                members[name] = Attribute(name, i == 0, INSTANCEATTR)
 
     if analyzer:
         # append instance attributes (cf. self.attr1) if analyzer knows
@@ -241,37 +229,27 @@ def get_object_members(subject: Any, objpath: List[str], attrgetter: Callable,
     return members
 
 
-class ClassAttribute:
-    """The attribute of the class."""
-
-    def __init__(self, cls: Any, name: str, value: Any, docstring: Optional[str] = None):
-        self.class_ = cls
-        self.name = name
-        self.value = value
-        self.docstring = docstring
-
-
 def get_class_members(subject: Any, objpath: List[str], attrgetter: Callable
-                      ) -> Dict[str, ClassAttribute]:
+                      ) -> Dict[str, "ObjectMember"]:
     """Get members and attributes of target class."""
-    from sphinx.ext.autodoc import INSTANCEATTR
+    from sphinx.ext.autodoc import INSTANCEATTR, ObjectMember
 
     # the members directly defined in the class
     obj_dict = attrgetter(subject, '__dict__', {})
 
-    members = {}  # type: Dict[str, ClassAttribute]
+    members: Dict[str, ObjectMember] = {}
 
     # enum members
     if isenumclass(subject):
         for name, value in subject.__members__.items():
             if name not in members:
-                members[name] = ClassAttribute(subject, name, value)
+                members[name] = ObjectMember(name, value, class_=subject)
 
         superclass = subject.__mro__[1]
         for name in obj_dict:
             if name not in superclass.__dict__:
                 value = safe_getattr(subject, name)
-                members[name] = ClassAttribute(subject, name, value)
+                members[name] = ObjectMember(name, value, class_=subject)
 
     # members in __slots__
     try:
@@ -280,68 +258,59 @@ def get_class_members(subject: Any, objpath: List[str], attrgetter: Callable
             from sphinx.ext.autodoc import SLOTSATTR
 
             for name, docstring in __slots__.items():
-                members[name] = ClassAttribute(subject, name, SLOTSATTR, docstring)
-    except (AttributeError, TypeError, ValueError):
+                members[name] = ObjectMember(name, SLOTSATTR, class_=subject,
+                                             docstring=docstring)
+    except (TypeError, ValueError):
         pass
 
     # other members
     for name in dir(subject):
         try:
             value = attrgetter(subject, name)
+            if ismock(value):
+                value = undecorate(value)
+
             unmangled = unmangle(subject, name)
             if unmangled and unmangled not in members:
                 if name in obj_dict:
-                    members[unmangled] = ClassAttribute(subject, unmangled, value)
+                    members[unmangled] = ObjectMember(unmangled, value, class_=subject)
                 else:
-                    members[unmangled] = ClassAttribute(None, unmangled, value)
+                    members[unmangled] = ObjectMember(unmangled, value)
         except AttributeError:
             continue
 
     try:
         for cls in getmro(subject):
-            # annotation only member (ex. attr: int)
-            try:
-                for name in getannotations(cls):
-                    name = unmangle(cls, name)
-                    if name and name not in members:
-                        members[name] = ClassAttribute(cls, name, INSTANCEATTR)
-            except AttributeError:
-                pass
-
-            # append instance attributes (cf. self.attr1) if analyzer knows
             try:
                 modname = safe_getattr(cls, '__module__')
                 qualname = safe_getattr(cls, '__qualname__')
                 analyzer = ModuleAnalyzer.for_module(modname)
                 analyzer.analyze()
+            except AttributeError:
+                qualname = None
+                analyzer = None
+            except PycodeError:
+                analyzer = None
+
+            # annotation only member (ex. attr: int)
+            for name in getannotations(cls):
+                name = unmangle(cls, name)
+                if name and name not in members:
+                    if analyzer and (qualname, name) in analyzer.attr_docs:
+                        docstring = '\n'.join(analyzer.attr_docs[qualname, name])
+                    else:
+                        docstring = None
+
+                    members[name] = ObjectMember(name, INSTANCEATTR, class_=cls,
+                                                 docstring=docstring)
+
+            # append instance attributes (cf. self.attr1) if analyzer knows
+            if analyzer:
                 for (ns, name), docstring in analyzer.attr_docs.items():
                     if ns == qualname and name not in members:
-                        members[name] = ClassAttribute(cls, name, INSTANCEATTR,
-                                                       '\n'.join(docstring))
-            except (AttributeError, PycodeError):
-                pass
+                        members[name] = ObjectMember(name, INSTANCEATTR, class_=cls,
+                                                     docstring='\n'.join(docstring))
     except AttributeError:
         pass
 
     return members
-
-
-from sphinx.ext.autodoc.mock import (MockFinder, MockLoader, _MockModule, _MockObject,  # NOQA
-                                     mock)
-
-deprecated_alias('sphinx.ext.autodoc.importer',
-                 {
-                     '_MockModule': _MockModule,
-                     '_MockObject': _MockObject,
-                     'MockFinder': MockFinder,
-                     'MockLoader': MockLoader,
-                     'mock': mock,
-                 },
-                 RemovedInSphinx40Warning,
-                 {
-                     '_MockModule': 'sphinx.ext.autodoc.mock._MockModule',
-                     '_MockObject': 'sphinx.ext.autodoc.mock._MockObject',
-                     'MockFinder': 'sphinx.ext.autodoc.mock.MockFinder',
-                     'MockLoader': 'sphinx.ext.autodoc.mock.MockLoader',
-                     'mock': 'sphinx.ext.autodoc.mock.mock',
-                 })
