@@ -1,12 +1,4 @@
-"""
-    sphinx.jinja2glue
-    ~~~~~~~~~~~~~~~~~
-
-    Glue code for the jinja2 templating engine.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Glue code for the jinja2 templating engine."""
 
 from os import path
 from pprint import pformat
@@ -23,7 +15,7 @@ from sphinx.util import logging
 from sphinx.util.osutil import mtimes_of_files
 
 try:
-    from jinja2.utils import pass_context  # type: ignore # jinja2-3.0 or above
+    from jinja2.utils import pass_context
 except ImportError:
     from jinja2 import contextfunction as pass_context
 
